@@ -5,24 +5,22 @@ import json
 import os
 import random
 import urllib
-from pathlib import Path
 
-import cv2
-import matplotlib.pyplot as plt
-import numpy as np
 import requests
 from PIL import Image
 
-from roboflow.config import OBJECT_DETECTION_MODEL
+from roboflow.config import OBJECT_DETECTION_MODEL, OBJECT_DETECTION_URL
+from roboflow.models.inference import InferenceModel
 from roboflow.util.image_utils import check_image_url
 from roboflow.util.prediction import PredictionGroup
-from roboflow.util.versions import (
-    print_warn_for_wrong_dependencies_versions,
-    warn_for_wrong_dependencies_versions,
-)
+from roboflow.util.versions import print_warn_for_wrong_dependencies_versions
 
 
-class ObjectDetectionModel:
+class ObjectDetectionModel(InferenceModel):
+    """
+    Run inference on an object detection model hosted on Roboflow or served through Roboflow Inference.
+    """  # noqa: E501 // docs
+
     def __init__(
         self,
         api_key,
@@ -40,33 +38,37 @@ class ObjectDetectionModel:
         preprocessing=None,
     ):
         """
-        From Roboflow Docs:
+        Create a ObjectDetectionModel object through which you can run inference.
 
-        :param api_key: Your API key (obtained via your workspace API settings page)
-        :param name: The url-safe version of the dataset name.  You can find it in the web UI by looking at
-        the URL on the main project view or by clicking the "Get curl command" button in the train results section of
-        your dataset version after training your model.
-        :param local: Address of the local server address if running a local Roboflow deployment server. ex. http://localhost:9001/
-        :param version: The version number identifying the version of of your dataset
-        :param classes: Restrict the predictions to only those of certain classes. Provide as a comma-separated string.
-        :param overlap: The maximum percentage (on a scale of 0-100) that bounding box predictions of the same class are
-        allowed to overlap before being combined into a single box.
-        :param confidence: A threshold for the returned predictions on a scale of 0-100. A lower number will return
-        more predictions. A higher number will return fewer high-certainty predictions
-        :param stroke: The width (in pixels) of the bounding box displayed around predictions (only has an effect when
-        format is image)
-        :param labels: Whether or not to display text labels on the predictions (only has an effect when format is
-        image).
-        :param format: json - returns an array of JSON predictions. (See response format tab).
-                       image - returns an image with annotated predictions as a binary blob with a Content-Type
-                       of image/jpeg.
-        """
+        Args:
+            api_key (str): Your API key (obtained via your workspace API settings page).
+            name (str): The url-safe version of the dataset name. You can find it in the web UI by looking at
+                        the URL on the main project view or by clicking the "Get curl command" button in the train
+                        results section of your dataset version after training your model.
+            local (str): Address of the local server address if running a local Roboflow deployment server.
+                        Ex. http://localhost:9001/
+            version (str): The version number identifying the version of your dataset.
+            classes (str): Restrict the predictions to only those of certain classes. Provide as a comma-separated string.
+            overlap (int): The maximum percentage (on a scale of 0-100) that bounding box predictions of the same class are
+                        allowed to overlap before being combined into a single box.
+            confidence (int): A threshold for the returned predictions on a scale of 0-100. A lower number will return
+                            more predictions. A higher number will return fewer high-certainty predictions.
+            stroke (int): The width (in pixels) of the bounding box displayed around predictions (only has an effect when
+                        format is image).
+            labels (bool): Whether or not to display text labels on the predictions (only has an effect when format is
+                        image).
+            format (str): The format of the output.
+                        - 'json': returns an array of JSON predictions (See response format tab).
+                        - 'image': returns an image with annotated predictions as a binary blob with a Content-Type
+                                    of image/jpeg.
+        """  # noqa: E501 // docs
         # Instantiate different API URL parameters
         # To be moved to predict
+        super().__init__(api_key, id)
         self.__api_key = api_key
         self.id = id
         self.name = name
-        self.version = version
+        self.version = version or self.version
         self.classes = classes
         self.overlap = overlap
         self.confidence = confidence
@@ -78,7 +80,7 @@ class ObjectDetectionModel:
 
         # local needs to be passed from Project
         if local is None:
-            self.base_url = "https://detect.roboflow.com/"
+            self.base_url = OBJECT_DETECTION_URL + "/"
         else:
             print("initalizing local object detection model hosted at :" + local)
             self.base_url = local
@@ -100,9 +102,12 @@ class ObjectDetectionModel:
         format=None,
     ):
         """
-        Loads a Model based on a Model Endpoint
+        Loads a Model from on a model endpoint.
 
-        :param model_endpoint: This is the endpoint that is loaded into the api_url
+        Args:
+            name (str): The url-safe version of the dataset name
+            version (str): The version number identifying the version of your dataset.
+            local (bool): Whether the model is hosted locally or on Roboflow
         """
         # To load a model manually, they must specify a dataset slug
         self.name = name
@@ -118,7 +123,7 @@ class ObjectDetectionModel:
             format=format,
         )
 
-    def predict(
+    def predict(  # type: ignore[override]
         self,
         image_path,
         hosted=False,
@@ -130,12 +135,26 @@ class ObjectDetectionModel:
         labels=False,
     ):
         """
-        Infers detections based on image from specified model and image path
+        Infers detections based on image from specified model and image path.
 
-        :param image_path: Path to image or image array (can be local or hosted)
-        :param hosted: If image located on a hosted server, hosted should be True
-        :param format: output format from this method
-        :return: PredictionGroup --> a group of predictions based on Roboflow JSON response
+        Args:
+            image_path (str): path to the image you'd like to perform prediction on
+            hosted (bool): whether the image you're providing is hosted on Roboflow
+            format (str): The format of the output.
+
+        Returns:
+            PredictionGroup Object
+
+        Example:
+            >>> import roboflow
+
+            >>> rf = roboflow.Roboflow(api_key="")
+
+            >>> project = rf.workspace().project("PROJECT_ID")
+
+            >>> model = project.version("1").model
+
+            >>> prediction = model.predict("YOUR_IMAGE.jpg")
         """
         # Generate url before predicting
         self.__generate_url(
@@ -148,24 +167,33 @@ class ObjectDetectionModel:
         )
 
         # Check if image exists at specified path or URL or is an array
-        if hasattr(image_path, "__len__") == True:
+        if hasattr(image_path, "__len__") is True:
             pass
         else:
             self.__exception_check(image_path_check=image_path)
 
-        resize = False
+        original_dimensions = None
+        should_resize = False
         # If image is local image
         if not hosted:
-            if type(image_path) is str:
+            import cv2
+            import numpy as np
+
+            should_resize = (
+                "resize" in self.preprocessing.keys() and "Stretch" in self.preprocessing["resize"]["format"]
+            )
+
+            if isinstance(image_path, str):
                 image = Image.open(image_path).convert("RGB")
                 dimensions = image.size
                 original_dimensions = copy.deepcopy(dimensions)
 
-                # Here we resize the image to the preprocessing settings before sending it over the wire
-                if "resize" in self.preprocessing.keys():
-                    if dimensions[0] > int(
-                        self.preprocessing["resize"]["width"]
-                    ) or dimensions[1] > int(self.preprocessing["resize"]["height"]):
+                # Here we resize the image to the preprocessing settings
+                # before sending it over the wire
+                if should_resize:
+                    if dimensions[0] > int(self.preprocessing["resize"]["width"]) or dimensions[1] > int(
+                        self.preprocessing["resize"]["height"]
+                    ):
                         image = image.resize(
                             (
                                 int(self.preprocessing["resize"]["width"]),
@@ -173,7 +201,6 @@ class ObjectDetectionModel:
                             )
                         )
                         dimensions = image.size
-                        resize = True
 
                 # Create buffer
                 buffered = io.BytesIO()
@@ -197,14 +224,15 @@ class ObjectDetectionModel:
                 retval, buffer = cv2.imencode(".jpg", image_path)
                 # Currently cv2.imencode does not properly return shape
                 dimensions = buffer.shape
-                img_str = base64.b64encode(buffer)
+                img_str = base64.b64encode(buffer)  # type: ignore[arg-type]
                 img_str = img_str.decode("ascii")
                 resp = requests.post(
                     self.api_url,
                     data=img_str,
                     headers={"Content-Type": "application/x-www-form-urlencoded"},
                 )
-                # Replace with dimensions variable once cv2.imencode shape solution is found
+                # Replace with dimensions variable once
+                # cv2.imencode shape solution is found
                 image_dims = {"width": "0", "height": "0"}
             else:
                 raise ValueError("image_path must be a string or a numpy array.")
@@ -220,36 +248,16 @@ class ObjectDetectionModel:
         if self.format == "json":
             resp_json = resp.json()
 
-            if resize:
+            if should_resize and original_dimensions is not None:
                 new_preds = []
                 for p in resp_json["predictions"]:
-                    p["x"] = int(
-                        p["x"]
-                        * (
-                            int(original_dimensions[0])
-                            / int(self.preprocessing["resize"]["width"])
-                        )
-                    )
-                    p["y"] = int(
-                        p["y"]
-                        * (
-                            int(original_dimensions[1])
-                            / int(self.preprocessing["resize"]["height"])
-                        )
-                    )
+                    p["x"] = int(p["x"] * (int(original_dimensions[0]) / int(self.preprocessing["resize"]["width"])))
+                    p["y"] = int(p["y"] * (int(original_dimensions[1]) / int(self.preprocessing["resize"]["height"])))
                     p["width"] = int(
-                        p["width"]
-                        * (
-                            int(original_dimensions[0])
-                            / int(self.preprocessing["resize"]["width"])
-                        )
+                        p["width"] * (int(original_dimensions[0]) / int(self.preprocessing["resize"]["width"]))
                     )
                     p["height"] = int(
-                        p["height"]
-                        * (
-                            int(original_dimensions[1])
-                            / int(self.preprocessing["resize"]["height"])
-                        )
+                        p["height"] * (int(original_dimensions[1]) / int(self.preprocessing["resize"]["height"]))
                     )
 
                     new_preds.append(p)
@@ -279,16 +287,18 @@ class ObjectDetectionModel:
         web_cam_res=(416, 416),
     ):
         """
-        Infers detections based on webcam feed from specified model
+        Infers detections based on webcam feed from specified model.
 
-        :param webcam_id: Webcam ID (default 0)
-        :param inference_engine_url: Inference engine address to use (default https://detect.roboflow.com)
-        :param in_jupyter: Whether or not to display the webcam within Jupyter notebook (default True)
-        :param confidence: Confidence threshold for detections
-        :param overlap: Overlap threshold for detections
-        :param stroke: Stroke width for bounding box
-        :param labels: Whether to show labels on bounding box
-        """
+        Args:
+            webcam_id (int): Webcam ID (default 0)
+            inference_engine_url (str): Inference engine address to use (default https://detect.roboflow.com)
+            within_jupyter (bool): Whether or not to display the webcam within Jupyter notebook (default True)
+            confidence (int): Confidence threshold for detections
+            overlap (int): Overlap threshold for detections
+            stroke (int): Stroke width for bounding box
+            labels (bool): Whether to show labels on bounding box
+        """  # noqa: E501 // docs
+        import cv2
 
         os.environ["OPENCV_VIDEOIO_PRIORITY_MSMF"] = "0"
 
@@ -301,23 +311,19 @@ class ObjectDetectionModel:
             inference_engine_url=inference_engine_url,
         )
 
-        def plot_one_box(
-            x, img, color=None, label=None, line_thickness=None, colors=None
-        ):
+        def plot_one_box(x, img, color=None, label=None, line_thickness=None, colors=None):
             # Plots one bounding box on image img
 
             self.colors = {} if colors is None else colors
 
-            if label in colors.keys() and label is not None:
-                color = colors[label]
+            if label in self.colors and label is not None:
+                color = self.colors[label]
                 color = color.lstrip("#")
                 color = tuple(int(color[i : i + 2], 16) for i in (0, 2, 4))
             else:
                 color = [random.randint(0, 255) for _ in range(3)]
 
-            tl = (
-                line_thickness or round(0.002 * (img.shape[0] + img.shape[1]) / 2) + 1
-            )  # line/font thickness
+            tl = line_thickness or round(0.002 * (img.shape[0] + img.shape[1]) / 2) + 1  # line/font thickness
 
             c1, c2 = (int(x[0]), int(x[1])), (int(x[2]), int(x[3]))
 
@@ -379,7 +385,7 @@ class ObjectDetectionModel:
         def view(button):
             while True:
                 if stopButton is not None:
-                    if stopButton.value == True:
+                    if stopButton.value is True:
                         break
                 else:
                     if cv2.waitKey(1) & 0xFF == ord("q"):  # quit when 'q' is pressed
@@ -391,7 +397,7 @@ class ObjectDetectionModel:
                 frame = cv2.flip(frame, 1)  # if your camera reverses your image
 
                 _, frame_upload = cv2.imencode(".jpeg", frame)
-                img_str = base64.b64encode(frame_upload)
+                img_str = base64.b64encode(frame_upload)  # type: ignore[arg-type]
                 img_str = img_str.decode("ascii")
 
                 # post frame to the Roboflow API
@@ -460,11 +466,10 @@ class ObjectDetectionModel:
             view(stopButton)
 
     def __exception_check(self, image_path_check=None):
-        # Check if Image path exists exception check (for both hosted URL and local image)
+        # Check if Image path exists exception check
+        # (for both hosted URL and local image)
         if image_path_check is not None:
-            if not os.path.exists(image_path_check) and not check_image_url(
-                image_path_check
-            ):
+            if not os.path.exists(image_path_check) and not check_image_url(image_path_check):
                 raise Exception("Image does not exist at " + image_path_check + "!")
 
     def __generate_url(
@@ -478,10 +483,13 @@ class ObjectDetectionModel:
         format=None,
         inference_engine_url=None,
     ):
+        """
+        Generate the URL to run inference on.
+        """
         # Reassign parameters if any parameters are changed
         if local is not None:
             if not local:
-                self.base_url = "https://detect.roboflow.com/"
+                self.base_url = OBJECT_DETECTION_URL + "/"
             else:
                 self.base_url = "http://localhost:9001/"
 
